
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "thingstack.h"

static void not_empty();

static thingstack_p stack=(void *)NULL;

static void not_empty(){
  if (stack==NULL){
    err_fatal("thingstack","attempt to pop empty stack");
  }
}

static thing_p new_thing(thing_type t){
  thing_p new;
  new=malloc(sizeof(thing_t));
  new->type=t;
  new->next=stack;
  stack=new;
  return new;
}

void push_mark(){
  (void)new_thing(THING_MARK);
}

int top_is_mark(){
  return (stack && stack->type==THING_MARK);
}


void push_string(char *str){
  thing_p thing;

  thing=new_thing(THING_STRING);
  thing->string=strdup(str);
}
 

void push_integer(char *str){
  thing_p thing=new_thing(THING_INTEGER);

  lrs_alloc_mp(thing->integer);
  atomp(str,thing->integer);

}


void push_rational(char *str){
  thing_p thing=new_thing(THING_RATIONAL);
  char numstr[BUFSIZ];
  char denstr[BUFSIZ];
  lrs_alloc_mp(thing->integer);
  lrs_alloc_mp(thing->denominator);
  atoaa(str,numstr,denstr);
  atomp(numstr,thing->integer);
  atomp(denstr,thing->denominator);

  /*  prat("pushed=",thing->integer,thing->denominator); */
}

void pop_rational(lrs_mp num,lrs_mp den){
  not_empty();
  assert(stack->type==THING_INTEGER || stack->type==THING_RATIONAL);

  copy(num,stack->integer);
  lrs_clear_mp(stack->integer);
  if (stack->type==THING_INTEGER){
    itomp(1,den);
  } else {
    copy(den,stack->denominator);
    lrs_clear_mp(stack->denominator);
  }
  /*  prat("popped=",num,den); */
  pop();
}

void pop_rational_vector(lrs_mp_vector num, lrs_mp_vector den, int n){
  int j;
  for (j=n-1; j>=0; j--){
    pop_rational(num[j],den[j]);
  }
}
		  
extern void pop_integer(lrs_mp a){
  
  assert(stack && stack->type==THING_INTEGER);
  copy(a,stack->integer);
  lrs_clear_mp(stack->integer);
  pop();
}


extern int pop_int(){
  int r;
  assert(stack && stack->type==THING_INTEGER);
  r=mptoi(stack->integer);
  lrs_clear_mp(stack->integer);
  pop();
  return r;
}
    
extern void pop(){
  thing_p dead=stack;
  stack=stack->next;
  free(dead);
}
void stack_dump(){
  thing_p p=stack;
  while(p){
    switch(p->type){
    case THING_INTEGER:
      fputs("INTEGER\t",stdout);
      pmp("",p->integer);
      break;
    case THING_RATIONAL:
      fputs("RATIONAL\t",stdout);
      break;
    case THING_STRING:
      printf("STRING\t%s\n",p->string);
      break;
    case THING_MARK:
      printf("MARK\n");
      break;
    }
    p=p->next;
  }

}
